#
# config.py - settings used in the blockerbug application
#
# Copyright 2012, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   Tim Flink <tflink@redhat.com>

import os
import sys


class Config(object):
    PRODUCTION = False
    DEBUG = True
    SECRET_KEY = None  # REPLACE THIS WITH A RANDOM STRING
    SQLALCHEMY_DATABASE_URI = 'sqlite://'
    BUGZILLA_URL = 'https://partner-bugzilla.redhat.com/'
    BUGZILLA_XMLRPC = BUGZILLA_URL + 'xmlrpc.cgi'
    KOJI_URL = 'http://koji.stg.fedoraproject.org/'
    BODHI_URL = 'https://bodhi.stg.fedoraproject.org/'
    FAS_ADMIN_GROUP = 'qa-admin'
    FAS_USER = ''
    FAS_PASSWORD = ''
    FAS_HTTPS_REQUIRED = False
    FAS_CHECK_CERT = False
    FAS_BASE_URL = 'https://admin.stg.fedoraproject.org/accounts/'
    LOGFILE = '/var/log/blockerbugs/blockerbugs.log'
    FILE_LOGGING = False
    SYSLOG_LOGGING = False
    STREAM_LOGGING = True
    SHOW_DB_URI = False
    BEHIND_PROXY = False
    FEDMENU_URL = ""
    FEDMENU_DATA_URL = ""
    PAGURE_URL = "https://stg.pagure.io/"
    PAGURE_API = "https://stg.pagure.io/api/0/"
    PAGURE_REPO = "fedoraqa-test"
    PAGURE_REPO_TOKEN = "YOUR SECRET API TOKEN FROM PROJECT SETTINGS"
    PAGURE_REPO_WEBHOOK_KEY = "YOUR WEBHOOK KEY PROJECT SETTINGS"
    PAGURE_BOT_USERNAME = 'blockerbot'
    PAGURE_BOT_ENABLED = True
    PAGURE_BOT_LOOP_THRESHOLD = 2
    PAGURE_DISCUSSION_TITLE = "$bugid $summary"
    PAGURE_DISCUSSION_CONTENT = '''\
Bug details: ** $bug_url **

#### Current vote summary
$vote_summary

To learn how to vote, see:
$pagure_url$pagure_repo
'''


class ProductionConfig(Config):
    PRODUCTION = True
    DEBUG = False
    BUGZILLA_URL = 'https://bugzilla.redhat.com/'
    BUGZILLA_XMLRPC = BUGZILLA_URL + 'xmlrpc.cgi'
    KOJI_URL = 'http://koji.fedoraproject.org/'
    BODHI_URL = 'https://bodhi.fedoraproject.org/'
    FAS_HTTPS_REQUIRED = True
    FAS_BASE_URL = 'https://admin.fedoraproject.org/accounts/'
    PAGURE_URL = "https://pagure.io/"
    PAGURE_API = "https://pagure.io/api/0/"
    PAGURE_REPO = "fedora-qa/blocker-review"
    SHOW_DB_URI = False


class DevelopmentConfig(Config):
    TRAP_BAD_REQUEST_ERRORS = True
    SQLALCHEMY_DATABASE_URI = 'sqlite:////var/tmp/blockerbugs_db.sqlite'
    SHOW_DB_URI = True


class TestingConfig(Config):
    TESTING = True


def openshift_config(config_object, openshift_production):
    # First, get db details from env
    try:
        config_object["SQLALCHEMY_DATABASE_URI"] = "postgresql+psycopg2://%s:%s@%s:%s/%s" % (
            os.environ["POSTGRESQL_USER"],
            os.environ["POSTGRESQL_PASSWORD"],
            os.environ["POSTGRESQL_SERVICE_HOST"],
            os.environ["POSTGRESQL_SERVICE_PORT"],
            os.environ["POSTGRESQL_DATABASE"]
        )
        config_object["SECRET_KEY"] = os.environ["SECRET_KEY"]
    except(KeyError):
        print("OpenShift mode enabled but required values couldn't be fetched. "
              "Check, if you have these variables defined in you env: "
              "(POSTGRESQL_[USER, PASSWORD, DATABASE, SERVICE_HOST, SERVICE_PORT], SECRET_KEY)", file=sys.stderr)
        sys.exit(1)

    # And then try to get more data from OpenShift env
    try:
        config_object["FAS_PASSWORD"] = os.environ["FAS_PASSWORD"]
        config_object["FAS_USER"] = os.environ["FAS_USER"]
        config_object["FAS_ADMIN_GROUP"] = os.environ["FAS_ADMIN_GROUP"]
        config_object["PAGURE_REPO_TOKEN"] = os.environ["PAGURE_REPO_TOKEN"]
        config_object["PAGURE_REPO_WEBHOOK_KEY"] = os.environ["PAGURE_REPO_WEBHOOK_KEY"]
        config_object["PAGURE_BOT_USERNAME"] = os.environ["PAGURE_BOT_USERNAME"]
    except(KeyError):
        print("Expected aditional data to be defined OpenShift env: "
              "FAS_PASSWORD, FAS_USER, FAS_ADMIN_GROUP, OPENSHIFT_PROD.", file=sys.stderr)
        # In production OpenShift, exit on missing additional details
        if openshift_production == "1":
            sys.exit(1)

    # Get some more data from OpenShift, if set
    if os.getenv("PAGURE_BOT_ENABLED") == 0:
        config_object["PAGURE_BOT_ENABLED"] = False
    if os.getenv("PAGURE_URL"):
        config_object["PAGURE_URL"] = os.getenv("PAGURE_URL")
    if os.getenv("PAGURE_API"):
        config_object["PAGURE_API"] = os.getenv("PAGURE_API")
