#
# errors.py - Various API errors
#
# Copyright 2013, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   Tim Flink <tflink@redhat.com>
#   Ilgiz Islamgulov <ilgizisl@gmail.com>
#

# First, try to import modern http.client and support also old one on EPEL 7/Python 2
try:
    import http.client as httplib
except ImportError:
    import httplib


class RestApiError(Exception):
    code = 1000
    name = 'UnknownError'
    message = 'An unknown error occurred.'
    detail = ''
    http_status_code = httplib.INTERNAL_SERVER_ERROR

    def __init__(self, **kwargs):
        self.detail = kwargs.pop('detail', '')
        self.message = self.message % (kwargs)
        super(RestApiError, self).__init__()

    def to_dict(self):
        data = {
            'error': {'code': self.code,
                      'name': self.name,
                      'message': self.message,
                      'detail': self.detail}
        }
        return data

    def __str__(self):
        return '%d (%s) - %s "%s"' % \
               (self.code, self.name, self.message, self.detail)


class ValidationError(RestApiError):
    def __init__(self, message):
        self.message = message

    code = 1001
    name = 'ValidationError'
    http_status_code = httplib.BAD_REQUEST


class NoSuchObjectError(RestApiError):
    code = 1002
    name = 'NoSuchObject'
    message = 'The specified %(obj_type)s does not exist'
    http_status_code = httplib.NOT_FOUND


class MalformedJSONError(RestApiError):
    code = 1003
    name = 'MalformedJSON'
    http_status_code = httplib.BAD_REQUEST
    message = 'The JSON you provided is not well-formed.'


class InvalidArgumentError(RestApiError):
    code = 1004
    name = 'InvalidArgument'
    message = 'The specified %(arg_name)s is invalid'
    http_status_code = httplib.BAD_REQUEST


class AuthFailedError(RestApiError):
    code = 1005
    name = 'AuthenticationFailed'
    http_status_code = httplib.FORBIDDEN
    message = 'The provided credentials does not match.'
