import requests
from string import Template

from blockerbugs import app
from blockerbugs.models.bug import Bug


class PagureAPIException(Exception):
    pass


def create_bug_discussion(bug):
    title = Template(app.config['PAGURE_DISCUSSION_TITLE']).safe_substitute(
        bugid=bug.bugid,
        summary=bug.summary)
    content = Template(app.config['PAGURE_DISCUSSION_CONTENT']).safe_substitute(
        bug_url=bug.url,
        vote_summary='Nobody voted yet.',
        pagure_url=app.config['PAGURE_URL'],
        pagure_repo=app.config['PAGURE_REPO'],)

    return create_issue(title, content)


def update_issue(issue_id, vote_summary):
    update_issue_url = app.config['PAGURE_API'] + app.config['PAGURE_REPO'] + '/issue/%s' % issue_id
    discussion_link = app.config['PAGURE_URL'] + app.config['PAGURE_REPO'] + "/issue/%s" % issue_id

    bug = Bug.query.filter_by(discussion_link=discussion_link).first()

    bug_url = app.config['BUGZILLA_URL'] + "/show_bug.cgi?id=%s" % bug.bugid
    title = Template(app.config['PAGURE_DISCUSSION_TITLE']).safe_substitute(
        bugid=bug.bugid,
        summary=bug.summary)
    content = Template(app.config['PAGURE_DISCUSSION_CONTENT']).safe_substitute(
        bug_url=bug_url,
        vote_summary=vote_summary,
        pagure_url=app.config['PAGURE_URL'],
        pagure_repo=app.config['PAGURE_REPO'],)
    data = {
        'title': title,
        'issue_content': content,
    }

    app.logger.debug('POST request to %s' % update_issue_url)

    resp = requests.post(update_issue_url,
                         headers={
                             'Authorization': 'token %s' % app.config['PAGURE_REPO_TOKEN']
                         },
                         data=data)
    if resp.status_code != 200:
        raise PagureAPIException('Unable to create issue, response code: %d, '
            'response text:\n%s' % (resp.status_code, resp.text))


def create_issue(title, content, tag=None):
    create_issue_url = app.config['PAGURE_API'] + app.config['PAGURE_REPO'] + '/new_issue'
    data = {
        'title': title,
        'issue_content': content,
    }
    if tag:
        data['tag'] = tag

    app.logger.debug('POST request to %s' % create_issue_url)
    resp = requests.post(create_issue_url,
                         headers={
                             'Authorization': 'token %s' % app.config['PAGURE_REPO_TOKEN']
                         },
                         data=data)
    if resp.status_code != 200:
        raise PagureAPIException('Unable to create issue, response code: %d, '
            'response text:\n%s' % (resp.status_code, resp.text))

    issue_id = resp.json()['issue']['id']
    issue_url = app.config['PAGURE_URL'] + app.config['PAGURE_REPO'] + "/issue/%s" % issue_id

    app.logger.debug('Created pagure issue: %s' % issue_url)

    return issue_url


def get_issue(issue_id):
    get_issue_url = app.config['PAGURE_API'] + app.config['PAGURE_REPO'] + '/issue/%s' % issue_id
    resp = requests.get(get_issue_url)

    app.logger.debug('GET request to %s' % get_issue_url)

    if resp.status_code != 200:
        raise PagureAPIException('Unable to get issue, response code: %d, '
            'response text:\n%s' % (resp.status_code, resp.text))

    return resp.json()


def get_issue_comments(issue_id):
    issue = get_issue(issue_id)

    return issue['comments']


def post_comment(issue_id, comment):
    post_comment_url = app.config['PAGURE_API'] + app.config['PAGURE_REPO'] + '/issue/%s/comment' % issue_id
    data = {
        'comment': comment
    }

    app.logger.debug('POST request to %s' % post_comment_url)

    resp = requests.post(post_comment_url,
                         headers={
                             'Authorization': 'token %s' % app.config['PAGURE_REPO_TOKEN']
                         },
                         data=data)
    if resp.status_code != 200:
        raise PagureAPIException('Unable to create issue, response code: %d, '
            'response text:\n%s' % (resp.status_code, resp.text))


def get_group(group):
    get_group_url = app.config['PAGURE_API'] + '/group/%s' % group
    resp = requests.get(get_group_url)

    app.logger.debug('GET request to %s' % get_group_url)

    if resp.status_code != 200:
        raise PagureAPIException('Unable to get group info, response code: %d, '
            'response text:\n%s' % (resp.status_code, resp.text))

    return resp.json()
